# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# PAYMENT FORM
class PaymentFormViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    PaymentFormViewSet
    -----------

    :action list: Return the payment form list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given payment form.
        `Profiles`: `superuser`, `staff`

    :action light: Return the payment form list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new payment form instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a payment form instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a payment form instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a payment form instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PaymentForm.objects.all()
    serializer_class = serializers.PaymentFormSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'PaymentForm{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a payment form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a payment form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PAYMENT INSTALLMENT
class PaymentInstallmentViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    PaymentInstallmentViewSet
    -----------

    :action list: Return the payment installment list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given payment installment.
        `Profiles`: `superuser`, `staff`

    :action light: Return the payment installment list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a payment installment instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a payment installment instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PaymentInstallment.objects.all()
    serializer_class = serializers.PaymentInstallmentSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'PaymentInstallment{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a payment installment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a payment installment instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
